# Generated by roxygen2: do not edit by hand

S3method("$",strict_list)
S3method("[",strict_list)
S3method("[[",strict_list)
S3method(format,orderly_comparison)
S3method(format,orderly_query)
S3method(print,orderly_cleanup_status)
S3method(print,orderly_comparison)
S3method(print,orderly_query)
S3method(print,orderly_query_explain)
export(orderly_artefact)
export(orderly_cleanup)
export(orderly_cleanup_status)
export(orderly_compare_packets)
export(orderly_comparison_explain)
export(orderly_config)
export(orderly_config_set)
export(orderly_copy_files)
export(orderly_dependency)
export(orderly_description)
export(orderly_example)
export(orderly_example_show)
export(orderly_gitignore_update)
export(orderly_hash_data)
export(orderly_hash_file)
export(orderly_init)
export(orderly_interactive_set_search_options)
export(orderly_list_src)
export(orderly_location_add)
export(orderly_location_add_http)
export(orderly_location_add_packit)
export(orderly_location_add_path)
export(orderly_location_fetch_metadata)
export(orderly_location_list)
export(orderly_location_pull)
export(orderly_location_pull_metadata)
export(orderly_location_pull_packet)
export(orderly_location_push)
export(orderly_location_remove)
export(orderly_location_rename)
export(orderly_metadata)
export(orderly_metadata_extract)
export(orderly_metadata_read)
export(orderly_migrate_source)
export(orderly_new)
export(orderly_parameters)
export(orderly_parse_expr)
export(orderly_parse_file)
export(orderly_plugin_add_metadata)
export(orderly_plugin_context)
export(orderly_plugin_register)
export(orderly_prune_orphans)
export(orderly_query)
export(orderly_query_explain)
export(orderly_resource)
export(orderly_run)
export(orderly_run_info)
export(orderly_search)
export(orderly_search_options)
export(orderly_shared_resource)
export(orderly_strict_mode)
export(orderly_validate_archive)
importFrom(R6,R6Class)
