# Generated by roxygen2: do not edit by hand

export(orderly_artefact)
export(orderly_cleanup)
export(orderly_cleanup_status)
export(orderly_compare_packets)
export(orderly_comparison_explain)
export(orderly_config)
export(orderly_config_set)
export(orderly_copy_files)
export(orderly_dependency)
export(orderly_description)
export(orderly_example)
export(orderly_example_show)
export(orderly_gitignore_update)
export(orderly_hash_data)
export(orderly_hash_file)
export(orderly_init)
export(orderly_interactive_set_search_options)
export(orderly_list_src)
export(orderly_location_add)
export(orderly_location_add_http)
export(orderly_location_add_packit)
export(orderly_location_add_path)
export(orderly_location_fetch_metadata)
export(orderly_location_list)
export(orderly_location_pull)
export(orderly_location_pull_metadata)
export(orderly_location_pull_packet)
export(orderly_location_push)
export(orderly_location_remove)
export(orderly_location_rename)
export(orderly_metadata)
export(orderly_metadata_extract)
export(orderly_metadata_read)
export(orderly_new)
export(orderly_parameters)
export(orderly_parse_expr)
export(orderly_parse_file)
export(orderly_plugin_add_metadata)
export(orderly_plugin_context)
export(orderly_plugin_register)
export(orderly_prune_orphans)
export(orderly_query)
export(orderly_query_explain)
export(orderly_resource)
export(orderly_run)
export(orderly_run_info)
export(orderly_search)
export(orderly_search_options)
export(orderly_shared_resource)
export(orderly_strict_mode)
export(orderly_validate_archive)
importFrom(orderly,orderly_artefact)
importFrom(orderly,orderly_cleanup)
importFrom(orderly,orderly_cleanup_status)
importFrom(orderly,orderly_compare_packets)
importFrom(orderly,orderly_comparison_explain)
importFrom(orderly,orderly_config)
importFrom(orderly,orderly_config_set)
importFrom(orderly,orderly_copy_files)
importFrom(orderly,orderly_dependency)
importFrom(orderly,orderly_description)
importFrom(orderly,orderly_example)
importFrom(orderly,orderly_example_show)
importFrom(orderly,orderly_gitignore_update)
importFrom(orderly,orderly_hash_data)
importFrom(orderly,orderly_hash_file)
importFrom(orderly,orderly_init)
importFrom(orderly,orderly_interactive_set_search_options)
importFrom(orderly,orderly_list_src)
importFrom(orderly,orderly_location_add)
importFrom(orderly,orderly_location_add_http)
importFrom(orderly,orderly_location_add_packit)
importFrom(orderly,orderly_location_add_path)
importFrom(orderly,orderly_location_fetch_metadata)
importFrom(orderly,orderly_location_list)
importFrom(orderly,orderly_location_pull)
importFrom(orderly,orderly_location_pull_metadata)
importFrom(orderly,orderly_location_pull_packet)
importFrom(orderly,orderly_location_push)
importFrom(orderly,orderly_location_remove)
importFrom(orderly,orderly_location_rename)
importFrom(orderly,orderly_metadata)
importFrom(orderly,orderly_metadata_extract)
importFrom(orderly,orderly_metadata_read)
importFrom(orderly,orderly_new)
importFrom(orderly,orderly_parameters)
importFrom(orderly,orderly_parse_expr)
importFrom(orderly,orderly_parse_file)
importFrom(orderly,orderly_plugin_add_metadata)
importFrom(orderly,orderly_plugin_context)
importFrom(orderly,orderly_plugin_register)
importFrom(orderly,orderly_prune_orphans)
importFrom(orderly,orderly_query)
importFrom(orderly,orderly_query_explain)
importFrom(orderly,orderly_resource)
importFrom(orderly,orderly_run)
importFrom(orderly,orderly_run_info)
importFrom(orderly,orderly_search)
importFrom(orderly,orderly_search_options)
importFrom(orderly,orderly_shared_resource)
importFrom(orderly,orderly_strict_mode)
importFrom(orderly,orderly_validate_archive)
