# Imported from outpack

* Schema version 0.1.1
* Imported on 2024-07-10 10:46:08.587491
* From outpack @ f521a290636c01c777d6550bca78e0a8e0f051f8 (main)

Do not make changes to files here, they will be overwritten
Run ./scripts/update_schemas to update
